@@ -36,8 +36,8 @@
  *    It should work properly on input images of any depth, with
  *    and without colormaps.  There are 7 sections.
  *
- *    Section 1. Test write/read with lossless compression, with
- *    and without colormaps.  The results are tested for equality.
+ *    Section 1. Test write/read with lossless and lossy compression, with
+ *    and without colormaps.  The lossless results are tested for equality.
  *
  *    Section 2. Test read/write to file with different tiff compressions.
  *
@@ -51,8 +51,10 @@
  *
  *    Section 7. Test header reading
  *
- *    This test is dependent on the following external libraries:
+ *    This test requires the following external I/O libraries
  *        libjpeg, libtiff, libpng, libz
+ *    and optionally tests these:
+ *        libwebp, libopenjp2, libgif
  */
 
 #include "allheaders.h"
@@ -115,10 +117,22 @@ L_REGPARAMS  *rp;
     fprintf(stderr, "Omitting libpng tests in ioformats_reg\n");
 #endif  /* !HAVE_LIBPNG || !HAVE_LIBZ */
 
+#if  !HAVE_LIBWEBP
+    fprintf(stderr, "Omitting libwebp tests in ioformats_reg\n");
+#endif  /* !HAVE_LIBWEBP */
+
+#if  !HAVE_LIBJP2K
+    fprintf(stderr, "Omitting libopenjp2 tests in ioformats_reg\n");
+#endif  /* !HAVE_LIBJP2K */
+
+#if  !HAVE_LIBGIF
+    fprintf(stderr, "Omitting libgif tests in ioformats_reg\n");
+#endif  /* !HAVE_LIBGIF */
+
     if (regTestSetup(argc, argv, &rp))
         return 1;
 
-    /* --------- Part 1: Test all lossless formats for r/w to file ---------*/
+    /* --------- Part 1: Test all formats for r/w to file ---------*/
 
     failure = FALSE;
     success = TRUE;
@@ -216,22 +230,22 @@ L_REGPARAMS  *rp;
             success = FALSE;
             continue;
         }
-	d = pixGetDepth(pix);
+        d = pixGetDepth(pix);
         fprintf(stderr, "%d bpp\n", d);
-	if (i == 0) {   /* 1 bpp */
+        if (i == 0) {   /* 1 bpp */
             pixWrite("/tmp/regout/junkg3.tif", pix, IFF_TIFF_G3);
             pixWrite("/tmp/regout/junkg4.tif", pix, IFF_TIFF_G4);
             pixWrite("/tmp/regout/junkrle.tif", pix, IFF_TIFF_RLE);
             pixWrite("/tmp/regout/junkpb.tif", pix, IFF_TIFF_PACKBITS);
-	    if (testcomp("/tmp/regout/junkg3.tif", pix, IFF_TIFF_G3))
+            if (testcomp("/tmp/regout/junkg3.tif", pix, IFF_TIFF_G3))
                 success = FALSE;
-	    if (testcomp("/tmp/regout/junkg4.tif", pix, IFF_TIFF_G4))
+            if (testcomp("/tmp/regout/junkg4.tif", pix, IFF_TIFF_G4))
                 success = FALSE;
-	    if (testcomp("/tmp/regout/junkrle.tif", pix, IFF_TIFF_RLE))
+            if (testcomp("/tmp/regout/junkrle.tif", pix, IFF_TIFF_RLE))
                 success = FALSE;
-	    if (testcomp("/tmp/regout/junkpb.tif", pix, IFF_TIFF_PACKBITS))
+            if (testcomp("/tmp/regout/junkpb.tif", pix, IFF_TIFF_PACKBITS))
                 success = FALSE;
-	}
+        }
         pixWrite("/tmp/regout/junklzw.tif", pix, IFF_TIFF_LZW);
         pixWrite("/tmp/regout/junkzip.tif", pix, IFF_TIFF_ZIP);
         pixWrite("/tmp/regout/junknon.tif", pix, IFF_TIFF);
@@ -241,7 +255,7 @@ L_REGPARAMS  *rp;
             success = FALSE;
         if (testcomp("/tmp/regout/junknon.tif", pix, IFF_TIFF))
             success = FALSE;
-	pixDestroy(&pix);
+        pixDestroy(&pix);
     }
     if (success)
         fprintf(stderr,
@@ -259,9 +273,9 @@ L_REGPARAMS  *rp;
             success = FALSE;
             continue;
         }
-	d = pixGetDepth(pix);
+        d = pixGetDepth(pix);
         fprintf(stderr, "%d bpp\n", d);
-	if (i == 0) {   /* 1 bpp */
+        if (i == 0) {   /* 1 bpp */
             pixWriteMemTiff(&data, &size, pix, IFF_TIFF_G3);
             nbytes = nbytesInFile("/tmp/regout/junkg3.tif");
             fprintf(stderr, "nbytes = %lu, size = %lu\n",
@@ -271,7 +285,7 @@ L_REGPARAMS  *rp;
             lept_free(data);
             pixWriteMemTiff(&data, &size, pix, IFF_TIFF_G4);
             nbytes = nbytesInFile("/tmp/regout/junkg4.tif");
-            fprintf(stderr, "nbytes = %lu, size = %lu\n", 
+            fprintf(stderr, "nbytes = %lu, size = %lu\n",
                     (unsigned long)nbytes, (unsigned long)size);
             pixt = pixReadMemTiff(data, size, 0);
             if (testcomp_mem(pix, &pixt, i, IFF_TIFF_G4)) success = FALSE;
@@ -288,12 +302,12 @@ L_REGPARAMS  *rp;
             lept_free(data);
             pixWriteMemTiff(&data, &size, pix, IFF_TIFF_PACKBITS);
             nbytes = nbytesInFile("/tmp/regout/junkpb.tif");
-            fprintf(stderr, "nbytes = %lu, size = %lu\n", 
+            fprintf(stderr, "nbytes = %lu, size = %lu\n",
                     (unsigned long)nbytes, (unsigned long)size);
             pixt = pixReadMemTiff(data, size, 0);
             if (testcomp_mem(pix, &pixt, i, IFF_TIFF_PACKBITS)) success = FALSE;
             lept_free(data);
-	}
+        }
         pixWriteMemTiff(&data, &size, pix, IFF_TIFF_LZW);
         pixt = pixReadMemTiff(data, size, 0);
         if (testcomp_mem(pix, &pixt, i, IFF_TIFF_LZW)) success = FALSE;
@@ -318,7 +332,6 @@ L_REGPARAMS  *rp;
             "\n  ******* Failure on at least one tiff r/w to memory ******\n\n");
     if (!success) failure = TRUE;
 
-
     /* ---------------- Part 4: Test non-tiff r/w to memory ---------------- */
 
     pixDisplayWrite(NULL, -1);
@@ -328,17 +341,21 @@ L_REGPARAMS  *rp;
             success = FALSE;
             continue;
         }
-	d = pixGetDepth(pix);
+        d = pixGetDepth(pix);
         sprintf(psname, "/tmp/regout/junkps.%d", d);
         fprintf(stderr, "%d bpp\n", d);
-        if (d != 16) {
-            if (test_writemem(pix, IFF_PNG, NULL)) success = FALSE;
-            if (test_writemem(pix, IFF_BMP, NULL)) success = FALSE;
-        }
         if (test_writemem(pix, IFF_PNM, NULL)) success = FALSE;
         if (test_writemem(pix, IFF_PS, psname)) success = FALSE;
-	if (d == 8 || d == 32)
+        if (d == 16) continue;
+        if (test_writemem(pix, IFF_PNG, NULL)) success = FALSE;
+        if (test_writemem(pix, IFF_BMP, NULL)) success = FALSE;
+        if (d != 32)
+            if (test_writemem(pix, IFF_GIF, NULL)) success = FALSE;
+        if (d == 8 || d == 32) {
             if (test_writemem(pix, IFF_JFIF_JPEG, NULL)) success = FALSE;
+            if (test_writemem(pix, IFF_JP2, NULL)) success = FALSE;
+            if (test_writemem(pix, IFF_WEBP, NULL)) success = FALSE;
+        }
         pixDestroy(&pix);
     }
     if (success)
@@ -570,10 +587,12 @@ test_writemem(PIX      *pixs,
               l_int32   format,
               char     *psfile)
 {
-l_uint8  *data = NULL;
-l_int32   same;
-size_t    size = 0;
-PIX      *pixd = NULL;
+l_uint8   *data = NULL;
+l_int32    same = TRUE;
+l_int32    ds, dd;
+l_float32  diff;
+size_t     size = 0;
+PIX       *pixd = NULL;
 
     if (format == IFF_PS) {
         pixWriteMemPS(&data, &size, pixs, NULL, 0, 1.0);
@@ -595,6 +614,18 @@ PIX      *pixd = NULL;
     if (format == IFF_TIFF)
         return 0;
 #endif  /* !HAVE_LIBTIFF */
+#if !HAVE_LIBWEBP
+    if (format == IFF_WEBP)
+        return 0;
+#endif  /* !HAVE_LIBWEBP */
+#if !HAVE_LIBJP2K
+    if (format == IFF_JP2)
+        return 0;
+#endif  /* !HAVE_LIBJP2K */
+#if !HAVE_LIBGIF
+    if (format == IFF_GIF)
+        return 0;
+#endif  /* !HAVE_LIBGIF */
 
     if (pixWriteMem(&data, &size, pixs, format)) {
         fprintf(stderr, "Mem write fail for format %d\n", format);
@@ -605,15 +636,35 @@ PIX      *pixd = NULL;
         lept_free(data);
         return 1;
     }
-    if (format == IFF_JFIF_JPEG) {
-        fprintf(stderr, "jpeg size = %lu\n", (unsigned long)size);
+
+    if (format == IFF_JFIF_JPEG || format == IFF_JP2 || format == IFF_WEBP) {
+        ds = pixGetDepth(pixs);
+        dd = pixGetDepth(pixd);
+        if (dd == 8) {
+            pixCompareGray(pixs, pixd, L_COMPARE_ABS_DIFF, 0, NULL, &diff,
+                           NULL, NULL);
+        } else if (ds == 32 && dd == 32) {
+            pixCompareRGB(pixs, pixd, L_COMPARE_ABS_DIFF, 0, NULL, &diff,
+                          NULL, NULL);
+        } else {
+            fprintf(stderr, "skipping: ds = %d, dd = %d, format = %d\n",
+                    ds, dd, format);
+            pixDestroy(&pixd);
+            return 0;
+        }
+
+/*        fprintf(stderr, "  size = %lu bytes; diff = %5.2f, format = %d\n",
+                (unsigned long)size, diff, format); */
+        if (diff > 8.0) {
+            same = FALSE;
+            fprintf(stderr, "Mem write/read fail for format %d, diff = %5.2f\n",
+                    format, diff);
+        }
         pixDisplayWrite(pixd, 1);
-        same = TRUE;
-    }
-    else {
+    } else {
         pixEqual(pixs, pixd, &same);
         if (!same)
-           fprintf(stderr, "Mem write/read fail for format %d\n", format);
+            fprintf(stderr, "Mem write/read fail for format %d\n", format);
     }
     pixDestroy(&pixd);
     lept_free(data);
