@@ -2,10 +2,14 @@
 #
 #  This testing wrapper was written by James Le Cuirot.
 #
-#  It runs:
+#  It runs all the programs in AUTO_REG_PROGS in Makefile.am
+#  when the command 'make check' is invoked.  This same set can
+#  be run by doing:
 #      alltests_reg generate
 #      alltests_reg compare
-#  when the command 'make check' is invoked
+#
+#  Ten of the tests require gnuplot.  These tests are skipped if
+#  gnuplot is not available.
 #
 #  The wrapper receives several parameters in this form:
 #      path/to/source/config/test-driver <TEST DRIVER ARGS> -- ./foo_reg
@@ -22,4 +26,13 @@
 #  the build directory.
 
 eval TEST=\${${#}}
+
+TEST_NAME="${TEST##*/}"
+TEST_NAME="${TEST_NAME%_reg*}"
+
+case "${TEST_NAME}" in
+    colormask|colorspace|dna|enhance|fpix1|kernel|nearline|projection|rankbin|rankhisto)
+        which gnuplot > /dev/null || which wgnuplot > /dev/null || exit 77 ;;
+esac
+
 exec ${@%${TEST}} /bin/sh -c "cd \"${srcdir}\" && \"${PWD}/\"${TEST} generate && \"${PWD}/\"${TEST} compare"
