@@ -68,6 +68,7 @@
  *           l_int32   boxaInsertBox()
  *           l_int32   boxaRemoveBox()
  *           l_int32   boxaRemoveBoxAndSave()
+ *           BOXA     *boxaSaveValid()
  *           l_int32   boxaInitFull()
  *           l_int32   boxaClear()
  *
@@ -997,6 +998,42 @@ BOX    **array;
 }
 
 
+/*!
+ *  boxaSaveValid()
+ *
+ *      Input:  boxa
+ *              copyflag (L_COPY or L_CLONE)
+ *      Return: boxad if OK, null on error
+ *
+ *  Notes:
+ *      (1) This makes a copy/clone of each valid box.
+ */
+BOXA *
+boxaSaveValid(BOXA    *boxas,
+              l_int32  copyflag)
+{
+l_int32  i, n;
+BOX     *box;
+BOXA    *boxad;
+
+    PROCNAME("boxaSaveValid");
+
+    if (!boxas)
+        return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if (copyflag != L_COPY && copyflag != L_CLONE)
+        return (BOXA *)ERROR_PTR("invalid copyflag", procName, NULL);
+
+    n = boxaGetCount(boxas);
+    boxad = boxaCreate(n);
+    for (i = 0; i < n; i++) {
+        if ((box = boxaGetValidBox(boxas, i, copyflag)) != NULL)
+            boxaAddBox(boxad, box, L_INSERT);
+    }
+
+    return boxad;
+}
+
+
 /*!
  *  boxaInitFull()
  *
@@ -1941,7 +1978,8 @@ BOXA  *boxa;
          * We are writing to file first, instead of reading from the memory
          * buffer, because the gnu extension fmemopen() is not available
          * with other runtimes. */
-    fp = tmpfile();
+    if ((fp = tmpfile()) == NULL)
+        return (BOXA *)ERROR_PTR("tmpfile stream not opened", procName, NULL);
     fwrite(data, 1, size, fp);
     rewind(fp);
     boxa = boxaReadStream(fp);
@@ -2035,16 +2073,17 @@ FILE    *fp;
     PROCNAME("boxaWriteMem");
 
     if (!pdata)
-        return ERROR_INT("&data not defined", procName, 1 );
+        return ERROR_INT("&data not defined", procName, 1);
     *pdata = NULL;
     if (!psize)
-        return ERROR_INT("&size not defined", procName, 1 );
+        return ERROR_INT("&size not defined", procName, 1);
     *psize = 0;
     if (!boxa)
-        return ERROR_INT("&boxa not defined", procName, 1 );
+        return ERROR_INT("&boxa not defined", procName, 1);
 
         /* Serialize: write to file and read serialized data back into memory */
-    fp = tmpfile();
+    if ((fp = tmpfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
     ret = boxaWriteStream(fp, boxa);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
