@@ -40,6 +40,9 @@
  *      are not too close to gray pixels.
  *         PIX       *pixMaskOverColorPixels()
  *
+ *      Generates mask over pixels within a prescribed cube in RGB space
+ *         PIX       *pixMaskOverColorRange()
+ *
  *      Finds the fraction of pixels with "color" that are not close to black
  *         l_int32    pixColorFraction()
  *
@@ -63,6 +66,9 @@
  *         l_int32    makeRGBIndexTables()
  *         l_int32    getRGBFromIndex()
  *
+ *      Identify images that have highlight (red) color
+ *         l_int32    pixHasHighlightRed()
+ *
  *  Color is tricky.  If we consider gray (r = g = b) to have no color
  *  content, how should we define the color content in each component
  *  of an arbitrary pixel, as well as the overall color magnitude?
@@ -308,40 +314,40 @@ PIXCMAP   *cmap;
  *
  *  Notes:
  *      (1) For an RGB image, a gray pixel is one where all three components
- *          are equal.  We define the amount of color in an RGB pixel by
- *          considering the absolute value of the differences between the
- *          three color components.  Consider the two largest
+ *          are equal.  We define the amount of color in an RGB pixel as
+ *          a function depending on the absolute value of the differences
+ *          between the three color components.  Consider the two largest
  *          of these differences.  The pixel component in common to these
  *          two differences is the color farthest from the other two.
- *          The color magnitude in an RGB pixel can be taken as:
- *              * the average of these two differences; i.e., the
+ *          The color magnitude in an RGB pixel can be taken as one
+ *          of these three definitions:
+ *            (a) The average of these two differences.  This is the
  *                average distance from the two components that are
- *                nearest to each other to the third component, or
- *              * the minimum value of these two differences; i.e., the
+ *                nearest to each other to the third component.
+ *            (b) The minimum value of these two differences.  This is
+ *                the intermediate value of the three distances between
+ *                component values.  Stated otherwise, it is the
  *                maximum over all components of the minimum distance
  *                from that component to the other two components.
- *          Even more simply, the color magnitude can be taken as
- *              * the maximum difference between component values
+ *            (c) The maximum difference between component values.
  *      (2) As an example, suppose that R and G are the closest in
- *          magnitude.  Then the color is determined as:
- *              * the average distance of B from these two; namely,
- *                (|B - R| + |B - G|) / 2, which can also be found
- *                from |B - (R + G) / 2|, or
- *              * the minimum distance of B from these two; namely,
- *                min(|B - R|, |B - G|).
- *              * the max(|B - R|, |B - G|)
- *      (3) The three numbers (rwhite, gwhite and bwhite) can be thought
+ *          magnitude.  Then the color is determined as either:
+ *            (a) The average distance of B from these two:
+ *                   (|B - R| + |B - G|) / 2
+ *            (b) The minimum distance of B from these two:
+ *                   min(|B - R|, |B - G|).
+ *            (c) The maximum distance of B from these two:
+ *                   max(|B - R|, |B - G|)
+ *      (3) The three methods for choosing the color magnitude from
+ *          the components are selected with these flags:
+ *            (a) L_MAX_DIFF_FROM_AVERAGE_2
+ *            (b) L_MAX_MIN_DIFF_FROM_2
+ *            (c) L_MAX_DIFF
+ *      (4) The three numbers (rwhite, gwhite and bwhite) can be thought
  *          of as the values in the image corresponding to white.
  *          They are used to compensate for an unbalanced color white point.
  *          They must either be all 0 or all non-zero.  To turn this
  *          off, set them all to 0.
- *      (4) We allow the following methods for choosing the color
- *          magnitude from the three components:
- *              * L_MAX_DIFF_FROM_AVERAGE_2
- *              * L_MAX_MIN_DIFF_FROM_2
- *              * L_MAX_DIFF
- *          These are described above in (1) and (2), as well as at
- *          the top of this file.
  */
 PIX *
 pixColorMagnitude(PIX     *pixs,
@@ -534,6 +540,70 @@ PIXCMAP   *cmap;
 }
 
 
+/* ----------------------------------------------------------------------- *
+ *      Generates a mask over pixels that have RGB color components        *
+ *      within the prescribed range (a cube in RGB color space)            *
+ * ----------------------------------------------------------------------- */
+/*!
+ *  pixMaskOverColorRange()
+ *
+ *      Input:  pixs  (32 bpp rgb or 8 bpp colormapped)
+ *              rmin, rmax (min and max allowed values for red component)
+ *              gmin, gmax
+ *              bmin, bmax
+ *      Return: pixd (1 bpp, mask over color pixels), or null on error
+ */
+PIX *
+pixMaskOverColorRange(PIX     *pixs,
+                      l_int32  rmin,
+                      l_int32  rmax,
+                      l_int32  gmin,
+                      l_int32  gmax,
+                      l_int32  bmin,
+                      l_int32  bmax)
+{
+l_int32    w, h, d, i, j, wpls, wpld;
+l_int32    rval, gval, bval;
+l_uint32  *datas, *datad, *lines, *lined;
+PIX       *pixc, *pixd;
+PIXCMAP   *cmap;
+
+    PROCNAME("pixMaskOverColorRange");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+
+    cmap = pixGetColormap(pixs);
+    if (!cmap && d != 32)
+        return (PIX *)ERROR_PTR("pixs not cmapped or 32 bpp", procName, NULL);
+    if (cmap)
+        pixc = pixRemoveColormap(pixs, REMOVE_CMAP_TO_FULL_COLOR);
+    else
+        pixc = pixClone(pixs);
+
+    pixd = pixCreate(w, h, 1);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+    datas = pixGetData(pixc);
+    wpls = pixGetWpl(pixc);
+    for (i = 0; i < h; i++) {
+        lines = datas + i * wpls;
+        lined = datad + i * wpld;
+        for (j = 0; j < w; j++) {
+            extractRGBValues(lines[j], &rval, &gval, &bval);
+            if (rval < rmin || rval > rmax) continue;
+            if (gval < gmin || gval > gmax) continue;
+            if (bval < bmin || bval > bmax) continue;
+            SET_DATA_BIT(lined, j);
+        }
+    }
+
+    pixDestroy(&pixc);
+    return pixd;
+}
+
+
 /* ----------------------------------------------------------------------- *
  *   Finds the fraction of pixels with "color" that are not close to black *
  * ----------------------------------------------------------------------- */
@@ -606,11 +676,11 @@ l_uint32  *data, *line;
 
     PROCNAME("pixColorFraction");
 
+    if (ppixfract) *ppixfract = 0.0;
+    if (pcolorfract) *pcolorfract = 0.0;
     if (!ppixfract || !pcolorfract)
-        return ERROR_INT("&pixfract and &colorfract not both defined",
+        return ERROR_INT("&pixfract and &colorfract not defined",
                          procName, 1);
-    *ppixfract = 0.0;
-    *pcolorfract = 0.0;
     if (!pixs || pixGetDepth(pixs) != 32)
         return ERROR_INT("pixs not defined or not 32 bpp", procName, 1);
 
@@ -713,7 +783,7 @@ NUMA    *na;
     if (factor < 1) factor = 1;
 
     pixGetDimensions(pixs, &w, &h, NULL);
-    mincount = (l_int32)(minfract * w * h);
+    mincount = (l_int32)(minfract * w * h * factor * factor);
     if ((na = pixGetGrayHistogram(pixs, factor)) == NULL)
         return ERROR_INT("na not made", procName, 1);
     ncolors = 2;  /* add in black and white */
@@ -989,7 +1059,8 @@ PIXCMAP   *cmap;
         *pncolors = sum;
         FREE(inta);
 
-        if (factor == 1 && ((cmap = pixGetColormap(pixs)) != NULL)) {
+        cmap = pixGetColormap(pixs);
+        if (cmap && factor == 1) {
             count = pixcmapGetCount(cmap);
             if (sum != count)
                 L_WARNING("colormap size %d differs from actual colors\n",
@@ -1366,3 +1437,95 @@ getRGBFromIndex(l_uint32  index,
     return 0;
 }
 
+
+/* ----------------------------------------------------------------------- *
+ *             Identify images that have highlight (red) color             *
+ * ----------------------------------------------------------------------- */
+/*!
+ *  pixHasHighlightRed()
+ *
+ *      Input:  pixs  (32 bpp rgb)
+ *              factor (subsampling; an integer >= 1; use 1 for all pixels)
+ *              fract (threshold fraction of all image pixels)
+ *              fthresh (threshold on a function of the components; typ. ~2.5)
+ *              &hasred (<return> 1 if red pixels are above threshold)
+ *              &ratio (<optional return> normalized fraction of threshold
+ *                      red pixels that is actually observed)
+ *              &pixdb (<optional return> seed pixel mask)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Pixels are identified as red if they satisfy two conditions:
+ *          (a) The components satisfy (R-B)/B > @fthresh   (red or dark fg)
+ *          (b) The red component satisfied R > 128  (red or light bg)
+ *          Masks are generated for (a) and (b), and the intersection
+ *          gives the pixels that are red but not either light bg or
+ *          dark fg.
+ *      (2) A typical value for fract = 0.0001, which gives sensitivity
+ *          to an image where a small fraction of the pixels are printed
+ *          in red.
+ *      (3) A typical value for fthresh = 2.5.  Higher values give less
+ *          sensitivity to red, and fewer false positives.
+ */
+l_int32
+pixHasHighlightRed(PIX        *pixs,
+                   l_int32     factor,
+                   l_float32   fract,
+                   l_float32   fthresh,
+                   l_int32    *phasred,
+                   l_float32  *pratio,
+                   PIX       **ppixdb)
+{
+l_int32    w, h, count;
+l_float32  ratio;
+PIX       *pix1, *pix2, *pix3, *pix4;
+FPIX      *fpix;
+
+    PROCNAME("pixHasHighlightRed");
+
+    if (pratio) *pratio = 0.0;
+    if (ppixdb) *ppixdb = NULL;
+    if (!phasred)
+        return ERROR_INT("&hasred not defined", procName, 1);
+    *phasred = 0;
+    if (!pixs || pixGetDepth(pixs) != 32)
+        return ERROR_INT("pixs not defined or not 32 bpp", procName, 1);
+    if (fthresh < 1.5 || fthresh > 3.5)
+        L_WARNING("fthresh = %f is out of normal bounds\n", procName, fthresh);
+
+    if (factor > 1)
+        pix1 = pixScaleByIntSampling(pixs, factor);
+    else
+        pix1 = pixClone(pixs);
+
+        /* Identify pixels that are either red or dark foreground */
+    fpix = pixComponentFunction(pix1, 1.0, 0.0, -1.0, 0.0, 0.0, 1.0);
+    pix2 = fpixThresholdToPix(fpix, fthresh);
+    pixInvert(pix2, pix2);
+
+        /* Identify pixels that are either red or light background */
+    pix3 = pixGetRGBComponent(pix1, COLOR_RED);
+    pix4 = pixThresholdToBinary(pix3, 130);
+    pixInvert(pix4, pix4);
+
+    pixAnd(pix4, pix4, pix2);
+    pixCountPixels(pix4, &count, NULL);
+    pixGetDimensions(pix4, &w, &h, NULL);
+    L_INFO("count = %d, thresh = %d\n", procName, count,
+           (l_int32)(fract * w * h));
+    ratio = (l_float32)count / (fract * w * h);
+    if (pratio) *pratio = ratio;
+    if (ratio >= 1.0)
+        *phasred = 1;
+    if (ppixdb)
+        *ppixdb = pix4;
+    else
+        pixDestroy(&pix4);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    pixDestroy(&pix3);
+    fpixDestroy(&fpix);
+    return 0;
+}
+
+
