@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -46,6 +57,10 @@
  *           PIX     *pixModifyHue()
  *           PIX     *pixModifySaturation()
  *           l_int32  pixMeasureSaturation()
+ *           PIX     *pixModifyBrightness()
+ *
+ *      Color shifting
+ *           PIX     *pixColorShiftRGB()
  *
  *      General multiplicative constant color transform
  *           PIX     *pixMultConstantColor()
@@ -94,8 +109,6 @@
  */
 
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <math.h>
 #include "allheaders.h"
 
@@ -167,7 +180,7 @@ PIXCMAP  *cmap;
     if (pixd && (pixd != pixs))
         return (PIX *)ERROR_PTR("pixd not null or pixs", procName, pixd);
     if (gamma <= 0.0) {
-        L_WARNING("gamma must be > 0.0; setting to 1.0", procName);
+        L_WARNING("gamma must be > 0.0; setting to 1.0\n", procName);
         gamma = 1.0;
     }
     if (minval >= maxval)
@@ -243,7 +256,7 @@ NUMA    *nag;
     if (minval >= maxval)
         return (PIX *)ERROR_PTR("minval not < maxval", procName, pixd);
     if (gamma <= 0.0) {
-        L_WARNING("gamma must be > 0.0; setting to 1.0", procName);
+        L_WARNING("gamma must be > 0.0; setting to 1.0\n", procName);
         gamma = 1.0;
     }
 
@@ -294,7 +307,7 @@ PIX   *pixalpha;
     if (pixd && (pixd != pixs))
         return (PIX *)ERROR_PTR("pixd not null or pixs", procName, pixd);
     if (gamma <= 0.0) {
-        L_WARNING("gamma must be > 0.0; setting to 1.0", procName);
+        L_WARNING("gamma must be > 0.0; setting to 1.0\n", procName);
         gamma = 1.0;
     }
     if (minval >= maxval)
@@ -302,15 +315,15 @@ PIX   *pixalpha;
 
     if (gamma == 1.0 && minval == 0 && maxval == 255)
         return pixCopy(pixd, pixs);
-
-    pixalpha = pixGetRGBComponent(pixs, L_ALPHA_CHANNEL);  /* save */
     if (!pixd)  /* start with a copy if not in-place */
         pixd = pixCopy(NULL, pixs);
 
+    pixalpha = pixGetRGBComponent(pixs, L_ALPHA_CHANNEL);  /* save */
     if ((nag = numaGammaTRC(gamma, minval, maxval)) == NULL)
         return (PIX *)ERROR_PTR("nag not made", procName, pixd);
     pixTRCMap(pixd, NULL, nag);
     pixSetRGBComponent(pixd, pixalpha, L_ALPHA_CHANNEL);  /* restore */
+    pixSetSpp(pixd, 4);
 
     numaDestroy(&nag);
     pixDestroy(&pixalpha);
@@ -349,7 +362,7 @@ NUMA      *na;
     if (minval >= maxval)
         return (NUMA *)ERROR_PTR("minval not < maxval", procName, NULL);
     if (gamma <= 0.0) {
-        L_WARNING("gamma must be > 0.0; setting to 1.0", procName);
+        L_WARNING("gamma must be > 0.0; setting to 1.0\n", procName);
         gamma = 1.0;
     }
 
@@ -419,7 +432,7 @@ PIXCMAP  *cmap;
     if (pixd && (pixd != pixs))
         return (PIX *)ERROR_PTR("pixd not null or pixs", procName, pixd);
     if (factor < 0.0) {
-        L_WARNING("factor must be >= 0.0; using 0.0", procName);
+        L_WARNING("factor must be >= 0.0; using 0.0\n", procName);
         factor = 0.0;
     }
     if (factor == 0.0)
@@ -488,7 +501,7 @@ NUMA    *nac;
         return (PIX *)ERROR_PTR("depth not 8 or 32 bpp", procName, pixd);
 
     if (factor < 0.0) {
-        L_WARNING("factor must be >= 0.0; using 0.0", procName);
+        L_WARNING("factor must be >= 0.0; using 0.0\n", procName);
         factor = 0.0;
     }
     if (factor == 0.0)
@@ -515,7 +528,7 @@ NUMA    *nac;
  *
  *  Notes:
  *      (1) The mapping is monotonic increasing, where 0 is mapped
- *          to 0 and 255 is mapped to 255. 
+ *          to 0 and 255 is mapped to 255.
  *      (2) As 'factor' is increased from 0.0 (where the mapping is linear),
  *          the map gets closer to its limit as a step function that
  *          jumps from 0 to 255 at the center (input value = 127).
@@ -530,7 +543,8 @@ NUMA      *na;
     PROCNAME("numaContrastTRC");
 
     if (factor < 0.0) {
-        L_WARNING("factor must be >= 0.0; using 0.0; no enhancement", procName);
+        L_WARNING("factor must be >= 0.0; using 0.0; no enhancement\n",
+                  procName);
         factor = 0.0;
     }
     if (factor == 0.0)
@@ -591,7 +605,7 @@ PIX *
 pixEqualizeTRC(PIX       *pixd,
                PIX       *pixs,
                l_float32  fract,
-	       l_int32    factor)
+               l_int32    factor)
 {
 l_int32   d;
 NUMA     *na;
@@ -615,7 +629,7 @@ PIXCMAP  *cmap;
 
     if (fract == 0.0)
         return pixCopy(pixd, pixs);
- 
+
         /* If there is a colormap, remove it. */
     if (cmap)
         pixt = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
@@ -631,8 +645,7 @@ PIXCMAP  *cmap;
         na = numaEqualizeTRC(pixd, fract, factor);
         pixTRCMap(pixd, NULL, na);
         numaDestroy(&na);
-    }
-    else {  /* 32 bpp */
+    } else {  /* 32 bpp */
         pix8 = pixGetRGBComponent(pixd, COLOR_RED);
         na = numaEqualizeTRC(pix8, fract, factor);
         pixTRCMap(pix8, NULL, na);
@@ -675,8 +688,8 @@ PIXCMAP  *cmap;
  */
 NUMA *
 numaEqualizeTRC(PIX       *pix,
-		l_float32  fract,
-		l_int32    factor)
+                l_float32  fract,
+                l_int32    factor)
 {
 l_int32    iin, iout, itarg;
 l_float32  val, sum;
@@ -694,7 +707,7 @@ NUMA      *nah, *nasum, *nad;
         return (NUMA *)ERROR_PTR("sampling factor < 1", procName, NULL);
 
     if (fract == 0.0)
-        L_WARNING("fract = 0.0; no equalization requested", procName);
+        L_WARNING("fract = 0.0; no equalization requested\n", procName);
 
     if ((nah = pixGetGrayHistogram(pix, factor)) == NULL)
         return (NUMA *)ERROR_PTR("histogram not made", procName, NULL);
@@ -779,8 +792,7 @@ l_uint32  *data, *datam, *line, *linem;
                     SET_DATA_BYTE(line, j, dval8);
                 }
             }
-        }
-        else {  /* d == 32 */
+        } else {  /* d == 32 */
             for (i = 0; i < h; i++) {
                 line = data + i * wpl;
                 for (j = 0; j < w; j++) {
@@ -793,8 +805,7 @@ l_uint32  *data, *datam, *line, *linem;
                 }
             }
         }
-    }
-    else {
+    } else {
         datam = pixGetData(pixm);
         wplm = pixGetWpl(pixm);
         pixGetDimensions(pixm, &wm, &hm, NULL);
@@ -814,8 +825,7 @@ l_uint32  *data, *datam, *line, *linem;
                     SET_DATA_BYTE(line, j, dval8);
                 }
             }
-        }
-        else {  /* d == 32 */
+        } else {  /* d == 32 */
             for (i = 0; i < h; i++) {
                 if (i >= hm)
                     break;
@@ -875,7 +885,7 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
     if (!pixs || (pixGetDepth(pixs) == 1))
         return (PIX *)ERROR_PTR("pixs not defined or 1 bpp", procName, NULL);
     if (fract <= 0.0 || halfwidth <= 0) {
-        L_WARNING("no sharpening requested; clone returned", procName);
+        L_WARNING("no sharpening requested; clone returned\n", procName);
         return pixClone(pixs);
     }
 
@@ -888,9 +898,9 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
 
         /* Sharpen */
     d = pixGetDepth(pixt);
-    if (d == 8)
+    if (d == 8) {
         pixd = pixUnsharpMaskingGray(pixt, halfwidth, fract);
-    else {  /* d == 32 */
+    } else {  /* d == 32 */
         pixr = pixGetRGBComponent(pixs, COLOR_RED);
         pixrs = pixUnsharpMaskingGray(pixr, halfwidth, fract);
         pixDestroy(&pixr);
@@ -904,6 +914,8 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
         pixDestroy(&pixrs);
         pixDestroy(&pixgs);
         pixDestroy(&pixbs);
+        if (pixGetSpp(pixs) == 4)
+            pixScaleAndTransferAlpha(pixd, pixs, 1.0, 1.0);
     }
 
     pixDestroy(&pixt);
@@ -944,7 +956,7 @@ PIXACC  *pixacc;
     if (d != 8 || pixGetColormap(pixs) != NULL)
         return (PIX *)ERROR_PTR("pixs not 8 bpp or has cmap", procName, NULL);
     if (fract <= 0.0 || halfwidth <= 0) {
-        L_WARNING("no sharpening requested; clone returned", procName);
+        L_WARNING("no sharpening requested; clone returned\n", procName);
         return pixClone(pixs);
     }
     if (halfwidth == 1 || halfwidth == 2)
@@ -1036,7 +1048,7 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
     if (!pixs || (pixGetDepth(pixs) == 1))
         return (PIX *)ERROR_PTR("pixs not defined or 1 bpp", procName, NULL);
     if (fract <= 0.0 || halfwidth <= 0) {
-        L_WARNING("no sharpening requested; clone returned", procName);
+        L_WARNING("no sharpening requested; clone returned\n", procName);
         return pixClone(pixs);
     }
     if (halfwidth != 1 && halfwidth != 2)
@@ -1051,9 +1063,9 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
 
         /* Sharpen */
     d = pixGetDepth(pixt);
-    if (d == 8)
+    if (d == 8) {
         pixd = pixUnsharpMaskingGrayFast(pixt, halfwidth, fract, direction);
-    else {  /* d == 32 */
+    } else {  /* d == 32 */
         pixr = pixGetRGBComponent(pixs, COLOR_RED);
         pixrs = pixUnsharpMaskingGrayFast(pixr, halfwidth, fract, direction);
         pixDestroy(&pixr);
@@ -1064,6 +1076,8 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
         pixbs = pixUnsharpMaskingGrayFast(pixb, halfwidth, fract, direction);
         pixDestroy(&pixb);
         pixd = pixCreateRGBImage(pixrs, pixgs, pixbs);
+        if (pixGetSpp(pixs) == 4)
+            pixScaleAndTransferAlpha(pixd, pixs, 1.0, 1.0);
         pixDestroy(&pixrs);
         pixDestroy(&pixgs);
         pixDestroy(&pixbs);
@@ -1104,7 +1118,7 @@ PIX  *pixd;
     if (pixGetDepth(pixs) != 8 || pixGetColormap(pixs) != NULL)
         return (PIX *)ERROR_PTR("pixs not 8 bpp or has cmap", procName, NULL);
     if (fract <= 0.0 || halfwidth <= 0) {
-        L_WARNING("no sharpening requested; clone returned", procName);
+        L_WARNING("no sharpening requested; clone returned\n", procName);
         return pixClone(pixs);
     }
     if (halfwidth != 1 && halfwidth != 2)
@@ -1156,7 +1170,7 @@ PIX       *pixd;
     if (d != 8 || pixGetColormap(pixs) != NULL)
         return (PIX *)ERROR_PTR("pixs not 8 bpp or has cmap", procName, NULL);
     if (fract <= 0.0 || halfwidth <= 0) {
-        L_WARNING("no sharpening requested; clone returned", procName);
+        L_WARNING("no sharpening requested; clone returned\n", procName);
         return pixClone(pixs);
     }
     if (halfwidth != 1 && halfwidth != 2)
@@ -1175,8 +1189,7 @@ PIX       *pixd;
         a[0] = -fract / 3.0;
         a[1] = 1.0 + fract * 2.0 / 3.0;
         a[2] = a[0];
-    }
-    else {  /* halfwidth == 2 */
+    } else {  /* halfwidth == 2 */
         a[0] = -fract / 5.0;
         a[1] = a[0];
         a[2] = 1.0 + fract * 4.0 / 5.0;
@@ -1198,8 +1211,7 @@ PIX       *pixd;
                     ival = L_MIN(255, ival);
                     SET_DATA_BYTE(lined, j, ival);
                 }
-            }
-            else {  /* halfwidth == 2 */
+            } else {  /* halfwidth == 2 */
                 for (j = 2; j < w - 2; j++) {
                     val = a[0] * GET_DATA_BYTE(lines, j - 2) +
                           a[1] * GET_DATA_BYTE(lines, j - 1) +
@@ -1213,8 +1225,7 @@ PIX       *pixd;
                 }
             }
         }
-    }
-    else {  /* direction == L_VERT */
+    } else {  /* direction == L_VERT */
         if (halfwidth == 1) {
             for (i = 1; i < h - 1; i++) {
                 lines0 = datas + (i - 1) * wpls;
@@ -1231,8 +1242,7 @@ PIX       *pixd;
                     SET_DATA_BYTE(lined, j, ival);
                 }
             }
-        }
-        else {  /* halfwidth == 2 */
+        } else {  /* halfwidth == 2 */
             for (i = 2; i < h - 2; i++) {
                 lines0 = datas + (i - 2) * wpls;
                 lines1 = datas + (i - 1) * wpls;
@@ -1293,7 +1303,7 @@ FPIX       *fpix;
     if (d != 8 || pixGetColormap(pixs) != NULL)
         return (PIX *)ERROR_PTR("pixs not 8 bpp or has cmap", procName, NULL);
     if (fract <= 0.0 || halfwidth <= 0) {
-        L_WARNING("no sharpening requested; clone returned", procName);
+        L_WARNING("no sharpening requested; clone returned\n", procName);
         return pixClone(pixs);
     }
     if (halfwidth != 1 && halfwidth != 2)
@@ -1407,6 +1417,7 @@ FPIX       *fpix;
  *          1.0 (or -1.0) represents a 360 degree rotation; i.e., no change.
  *      (2) If no modification is requested (fract = -1.0 or 0 or 1.0),
  *          return a copy unless in-place, in which case this is a no-op.
+ *      (3) See discussion of color-modification methods, in coloring.c.
  */
 PIX  *
 pixModifyHue(PIX       *pixd,
@@ -1426,7 +1437,7 @@ l_uint32  *data, *line;
     if (pixd && (pixd != pixs))
         return (PIX *)ERROR_PTR("pixd not null or pixs", procName, pixd);
     pixGetDimensions(pixs, &w, &h, &d);
-    if (d != 32) 
+    if (d != 32)
         return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
     if (L_ABS(fract) > 1.0)
         return (PIX *)ERROR_PTR("fract not in [-1.0 ... 1.0]", procName, NULL);
@@ -1435,7 +1446,7 @@ l_uint32  *data, *line;
 
     delhue = (l_int32)(240 * fract);
     if (delhue == 0 || delhue == 240 || delhue == -240) {
-        L_WARNING("no change requested in hue", procName);
+        L_WARNING("no change requested in hue\n", procName);
         return pixd;
     }
     if (delhue < 0)
@@ -1453,6 +1464,8 @@ l_uint32  *data, *line;
             composeRGBPixel(rval, gval, bval, line + j);
         }
     }
+    if (pixGetSpp(pixs) == 4)
+        pixScaleAndTransferAlpha(pixd, pixs, 1.0, 1.0);
 
     return pixd;
 }
@@ -1475,6 +1488,7 @@ l_uint32  *data, *line;
  *          saturation to 0 (255).
  *      (2) If fract = 0, no modification is requested; return a copy
  *          unless in-place, in which case this is a no-op.
+ *      (3) See discussion of color-modification methods, in coloring.c.
  */
 PIX  *
 pixModifySaturation(PIX       *pixd,
@@ -1490,14 +1504,14 @@ l_uint32  *data, *line;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     pixGetDimensions(pixs, &w, &h, &d);
-    if (d != 32) 
+    if (d != 32)
         return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
     if (L_ABS(fract) > 1.0)
         return (PIX *)ERROR_PTR("fract not in [-1.0 ... 1.0]", procName, NULL);
 
     pixd = pixCopy(pixd, pixs);
     if (fract == 0.0) {
-        L_WARNING("no change requested in saturation", procName);
+        L_WARNING("no change requested in saturation\n", procName);
         return pixd;
     }
 
@@ -1516,6 +1530,8 @@ l_uint32  *data, *line;
             composeRGBPixel(rval, gval, bval, line + j);
         }
     }
+    if (pixGetSpp(pixs) == 4)
+        pixScaleAndTransferAlpha(pixd, pixs, 1.0, 1.0);
 
     return pixd;
 }
@@ -1546,7 +1562,7 @@ l_uint32  *data, *line;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     pixGetDimensions(pixs, &w, &h, &d);
-    if (d != 32) 
+    if (d != 32)
         return ERROR_INT("pixs not 32 bpp", procName, 1);
     if (factor < 1)
         return ERROR_INT("subsampling factor < 1", procName, 1);
@@ -1569,6 +1585,169 @@ l_uint32  *data, *line;
 }
 
 
+/*!
+ *  pixModifyBrightness()
+ *
+ *      Input:  pixd (<optional> can be null, existing or equal to pixs)
+ *              pixs (32 bpp rgb)
+ *              fract (between -1.0 and 1.0)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) If fract > 0.0, it gives the fraction that the v-parameter,
+ *          which is max(r,g,b), is moved from its initial value toward 255.
+ *          If fract < 0.0, it gives the fraction that the v-parameter
+ *          is moved from its initial value toward 0.
+ *          The limiting values for fract = -1.0 (1.0) thus set the
+ *          v-parameter to 0 (255).
+ *      (2) If fract = 0, no modification is requested; return a copy
+ *          unless in-place, in which case this is a no-op.
+ *      (3) See discussion of color-modification methods, in coloring.c.
+ */
+PIX  *
+pixModifyBrightness(PIX       *pixd,
+                    PIX       *pixs,
+                    l_float32  fract)
+{
+l_int32    w, h, d, i, j, wpl;
+l_int32    rval, gval, bval, hval, sval, vval;
+l_uint32  *data, *line;
+
+    PROCNAME("pixModifyBrightness");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 32)
+        return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
+    if (L_ABS(fract) > 1.0)
+        return (PIX *)ERROR_PTR("fract not in [-1.0 ... 1.0]", procName, NULL);
+
+    pixd = pixCopy(pixd, pixs);
+    if (fract == 0.0) {
+        L_WARNING("no change requested in brightness\n", procName);
+        return pixd;
+    }
+
+    data = pixGetData(pixd);
+    wpl = pixGetWpl(pixd);
+    for (i = 0; i < h; i++) {
+        line = data + i * wpl;
+        for (j = 0; j < w; j++) {
+            extractRGBValues(line[j], &rval, &gval, &bval);
+            convertRGBToHSV(rval, gval, bval, &hval, &sval, &vval);
+            if (fract > 0.0)
+                vval = (l_int32)(vval + fract * (255.0 - vval));
+            else
+                vval = (l_int32)(vval * (1.0 + fract));
+            convertHSVToRGB(hval, sval, vval, &rval, &gval, &bval);
+            composeRGBPixel(rval, gval, bval, line + j);
+        }
+    }
+    if (pixGetSpp(pixs) == 4)
+        pixScaleAndTransferAlpha(pixd, pixs, 1.0, 1.0);
+
+    return pixd;
+}
+
+
+/*-----------------------------------------------------------------------*
+ *                             Color shifting                            *
+ *-----------------------------------------------------------------------*/
+/*!
+ *  pixColorShiftRGB()
+ *
+ *      Input:  pixs (32 bpp rgb)
+ *              rfract (fractional shift in red component)
+ *              gfract (fractional shift in green component)
+ *              bfract (fractional shift in blue component)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) This allows independent fractional shifts of the r,g and b
+ *          components.  A positive shift pushes to saturation (255);
+ *          a negative shift pushes toward 0 (black).
+ *      (2) The effect can be imagined using a color wheel that consists
+ *          (for our purposes) of these 6 colors, separated by 60 degrees:
+ *             red, magenta, blue, cyan, green, yellow
+ *      (3) So, for example, a negative shift of the blue component
+ *          (bfract < 0) could be accompanied by positive shifts
+ *          of red and green to make an image more yellow.
+ *      (4) Examples of limiting cases:
+ *            rfract = 1 ==> r = 255
+ *            rfract = -1 ==> r = 0
+ */
+PIX *
+pixColorShiftRGB(PIX       *pixs,
+                 l_float32  rfract,
+                 l_float32  gfract,
+                 l_float32  bfract)
+{
+l_int32    w, h, i, j, wpls, wpld, rval, gval, bval;
+l_int32   *rlut, *glut, *blut;
+l_uint32  *datas, *datad, *lines, *lined;
+l_float32  fi;
+PIX       *pixd;
+
+    PROCNAME("pixColorShiftRGB");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (pixGetDepth(pixs) != 32)
+        return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
+    if (rfract < -1.0 || rfract > 1.0)
+        return (PIX *)ERROR_PTR("rfract not in [-1.0,...,1.0]", procName, NULL);
+    if (gfract < -1.0 || gfract > 1.0)
+        return (PIX *)ERROR_PTR("gfract not in [-1.0,...,1.0]", procName, NULL);
+    if (bfract < -1.0 || bfract > 1.0)
+        return (PIX *)ERROR_PTR("bfract not in [-1.0,...,1.0]", procName, NULL);
+    if (rfract == 0.0 && gfract == 0.0 && bfract == 0.0)
+        return pixCopy(NULL, pixs);
+
+    rlut = (l_int32 *)CALLOC(256, sizeof(l_int32));
+    glut = (l_int32 *)CALLOC(256, sizeof(l_int32));
+    blut = (l_int32 *)CALLOC(256, sizeof(l_int32));
+    for (i = 0; i < 256; i++) {
+        fi = i;
+        if (rfract >= 0) {
+            rlut[i] = (l_int32)(fi + (255.0 - fi) * rfract);
+        } else {
+            rlut[i] = (l_int32)(fi * (1.0 + rfract));
+        }
+        if (gfract >= 0) {
+            glut[i] = (l_int32)(fi + (255.0 - fi) * gfract);
+        } else {
+            glut[i] = (l_int32)(fi * (1.0 + gfract));
+        }
+        if (bfract >= 0) {
+            blut[i] = (l_int32)(fi + (255.0 - fi) * bfract);
+        } else {
+            blut[i] = (l_int32)(fi * (1.0 + bfract));
+        }
+    }
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    pixd = pixCreate(w, h, 32);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+    for (i = 0; i < h; i++) {
+        lines = datas + i * wpls;
+        lined = datad + i * wpld;
+        for (j = 0; j < w; j++) {
+            extractRGBValues(lines[j], &rval, &gval, &bval);
+            composeRGBPixel(rlut[rval], glut[gval], blut[bval], lined + j);
+        }
+    }
+
+    FREE(rlut);
+    FREE(glut);
+    FREE(blut);
+    return pixd;
+}
+
+
 /*-----------------------------------------------------------------------*
  *            General multiplicative constant color transform            *
  *-----------------------------------------------------------------------*/
@@ -1833,4 +2012,3 @@ PIX     *pixg, *pixacc, *pixc1, *pixc2;
     pixDestroy(&pixc2);
     return pixc1;
 }
-
