@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -49,14 +60,15 @@
  *
  *    FPixa addition
  *          l_int32        fpixaAddFPix()
- *          l_int32        fpixaExtendArray()
- *          l_int32        fpixaExtendArrayToSize()
+ *          static l_int32 fpixaExtendArray()
+ *          static l_int32 fpixaExtendArrayToSize()
  *
  *    FPixa accessors
  *          l_int32        fpixaGetCount()
  *          l_int32        fpixaChangeRefcount()
  *          FPIX          *fpixaGetFPix()
  *          l_int32        fpixaGetFPixDimensions()
+ *          l_float32     *fpixaGetData()
  *          l_int32        fpixaGetPixel()
  *          l_int32        fpixaSetPixel()
  *
@@ -96,7 +108,7 @@
  *          l_int32        dpixWrite()
  *          l_int32        dpixWriteStream()
  *          DPIX          *dpixEndianByteSwap()
- * 
+ *
  *    Print FPix (subsampled, for debugging)
  *          l_int32        fpixPrintStream()
  */
@@ -106,6 +118,10 @@
 
 static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;   /* must be > 0 */
 
+    /* Static functions */
+static l_int32 fpixaExtendArray(FPIXA *fpixa);
+static l_int32 fpixaExtendArrayToSize(FPIXA *fpixa, l_int32 size);
+
 
 /*--------------------------------------------------------------------*
  *                     FPix Create/copy/destroy                       *
@@ -126,6 +142,7 @@ fpixCreate(l_int32  width,
            l_int32  height)
 {
 l_float32  *data;
+l_uint64    bignum;
 FPIX       *fpixd;
 
     PROCNAME("fpixCreate");
@@ -135,10 +152,17 @@ FPIX       *fpixd;
     if (height <= 0)
         return (FPIX *)ERROR_PTR("height must be > 0", procName, NULL);
 
+        /* Avoid overflow in malloc arg, malicious or otherwise */
+    bignum = 4L * width * height;   /* max number of bytes requested */
+    if (bignum > ((1LL << 31) - 1)) {
+        L_ERROR("requested w = %d, h = %d\n", procName, width, height);
+        return (FPIX *)ERROR_PTR("requested bytes >= 2^31", procName, NULL);
+    }
+
     if ((fpixd = (FPIX *)CALLOC(1, sizeof(FPIX))) == NULL)
         return (FPIX *)ERROR_PTR("CALLOC fail for fpixd", procName, NULL);
     fpixSetDimensions(fpixd, width, height);
-    fpixSetWpl(fpixd, width);
+    fpixSetWpl(fpixd, width);  /* 4-byte words */
     fpixd->refcount = 1;
 
     data = (l_float32 *)CALLOC(width * height, sizeof(l_float32));
@@ -331,7 +355,7 @@ FPIX       *fpix;
     PROCNAME("fpixDestroy");
 
     if (!pfpix) {
-        L_WARNING("ptr address is null!", procName);
+        L_WARNING("ptr address is null!\n", procName);
         return;
     }
 
@@ -404,7 +428,7 @@ fpixGetWpl(FPIX  *fpix)
     PROCNAME("fpixGetWpl");
 
     if (!fpix)
-        return ERROR_INT("fpix not defined", procName, 1);
+        return ERROR_INT("fpix not defined", procName, UNDEF);
     return fpix->wpl;
 }
 
@@ -683,7 +707,7 @@ FPIXA   *fpixa;
     PROCNAME("fpixaDestroy");
 
     if (pfpixa == NULL) {
-        L_WARNING("ptr address is NULL!", procName);
+        L_WARNING("ptr address is NULL!\n", procName);
         return;
     }
 
@@ -760,7 +784,7 @@ FPIX    *fpixc;
  *  Notes:
  *      (1) Doubles the size of the fpixa ptr array.
  */
-l_int32
+static l_int32
 fpixaExtendArray(FPIXA  *fpixa)
 {
     PROCNAME("fpixaExtendArray");
@@ -781,7 +805,7 @@ fpixaExtendArray(FPIXA  *fpixa)
  *  Notes:
  *      (1) If necessary, reallocs new fpixa ptrs array to @size.
  */
-l_int32
+static l_int32
 fpixaExtendArrayToSize(FPIXA   *fpixa,
                        l_int32  size)
 {
@@ -902,6 +926,36 @@ FPIX  *fpix;
 }
 
 
+/*!
+ *  fpixaGetData()
+ *
+ *      Input:  fpixa
+ *              index (into fpixa array)
+ *      Return: data (not a copy), or null on error
+ */
+l_float32 *
+fpixaGetData(FPIXA      *fpixa,
+             l_int32     index)
+{
+l_int32     n;
+l_float32  *data;
+FPIX       *fpix;
+
+    PROCNAME("fpixaGetData");
+
+    if (!fpixa)
+        return (l_float32 *)ERROR_PTR("fpixa not defined", procName, NULL);
+    n = fpixaGetCount(fpixa);
+    if (index < 0 || index >= n)
+        return (l_float32 *)ERROR_PTR("invalid index", procName, NULL);
+
+    fpix = fpixaGetFPix(fpixa, index, L_CLONE);
+    data = fpixGetData(fpix);
+    fpixDestroy(&fpix);
+    return data;
+}
+
+
 /*!
  *  fpixaGetPixel()
  *
@@ -992,6 +1046,7 @@ dpixCreate(l_int32  width,
            l_int32  height)
 {
 l_float64  *data;
+l_uint64    bignum;
 DPIX       *dpix;
 
     PROCNAME("dpixCreate");
@@ -1001,10 +1056,17 @@ DPIX       *dpix;
     if (height <= 0)
         return (DPIX *)ERROR_PTR("height must be > 0", procName, NULL);
 
+        /* Avoid overflow in malloc arg, malicious or otherwise */
+    bignum = 8L * width * height;   /* max number of bytes requested */
+    if (bignum > ((1LL << 31) - 1)) {
+        L_ERROR("requested w = %d, h = %d\n", procName, width, height);
+        return (DPIX *)ERROR_PTR("requested bytes >= 2^31", procName, NULL);
+    }
+
     if ((dpix = (DPIX *)CALLOC(1, sizeof(DPIX))) == NULL)
         return (DPIX *)ERROR_PTR("CALLOC fail for dpix", procName, NULL);
     dpixSetDimensions(dpix, width, height);
-    dpixSetWpl(dpix, width);  /* Note: 8 byte words here */
+    dpixSetWpl(dpix, width);  /* 8 byte words */
     dpix->refcount = 1;
 
     data = (l_float64 *)CALLOC(width * height, sizeof(l_float64));
@@ -1161,7 +1223,7 @@ l_float64  *data;
         return 0;
 
     dpixSetDimensions(dpixd, ws, hs);
-    dpixSetWpl(dpixd, ws);  /* Note: 8 byte words */
+    dpixSetWpl(dpixd, ws);  /* 8 byte words */
     bytes = 8 * ws * hs;
     data = dpixGetData(dpixd);
     if (data) FREE(data);
@@ -1191,7 +1253,7 @@ DPIX       *dpix;
     PROCNAME("dpixDestroy");
 
     if (!pdpix) {
-        L_WARNING("ptr address is null!", procName);
+        L_WARNING("ptr address is null!\n", procName);
         return;
     }
 
@@ -1587,7 +1649,7 @@ FPIX       *fpixt;
     fpixGetDimensions(fpixt, &w, &h);
     data = fpixGetData(fpixt);
     nbytes = w * h * sizeof(l_float32);
-    fpixGetResolution(fpixt, &xres, &yres); 
+    fpixGetResolution(fpixt, &xres, &yres);
     fprintf(fp, "\nFPix Version %d\n", FPIX_VERSION_NUMBER);
     fprintf(fp, "w = %d, h = %d, nbytes = %d\n", w, h, nbytes);
     fprintf(fp, "xres = %d, yres = %d\n", xres, yres);
@@ -1621,7 +1683,7 @@ fpixEndianByteSwap(FPIX  *fpixd,
                    FPIX  *fpixs)
 {
     PROCNAME("fpixEndianByteSwap");
-        
+
     if (!fpixs)
         return (FPIX *)ERROR_PTR("fpixs not defined", procName, fpixd);
     if (fpixd && (fpixs != fpixd))
@@ -1833,7 +1895,7 @@ dpixEndianByteSwap(DPIX  *dpixd,
                    DPIX  *dpixs)
 {
     PROCNAME("dpixEndianByteSwap");
-        
+
     if (!dpixs)
         return (DPIX *)ERROR_PTR("dpixs not defined", procName, dpixd);
     if (dpixd && (dpixs != dpixd))
