@@ -38,18 +38,24 @@
  *          PIX        *pixReadMemGif()
  *          l_int32     pixWriteMemGif()
  *
- *    This uses the gif library, version 4.1.6 or later.
- *    Do not use 4.1.4.  It has serious problems handling 1 bpp images.
- *
  *    The initial version of this module was generously contribued by
  *    Antony Dovgal.  He can be contacted at:  tony *AT* daylessday.org
  *
- *    There are some issues with version 5:
- *    - valgrind detects uninitialized values used used for writing
- *      and conditionally jumping in EGifPutScreenDesc().
- *    - DGifSlurp() crashes on some images, apparently triggered by
- *      by some GIF extension records.  The latter problem has been
- *      reported but not resolved as of October 2013.
+ *    Important version information:
+ *
+ *    (1) This uses the gif library, version 4.1.6 or later.
+ *        Do not use 4.1.4.  It has serious problems handling 1 bpp images.
+ *
+ *    (2) There are some issues with version 5.0:
+ *        - valgrind detects uninitialized values used used for writing
+ *          and conditionally jumping in EGifPutScreenDesc().
+ *        - DGifSlurp() crashes on some images, apparently triggered by
+ *          by some GIF extension records.  The latter problem has been
+ *          reported but not resolved as of October 2013.
+ *
+ *    (3) E. Raymond has been changing the high-level interface with 5.0
+ *        and 5.1, and to keep up we have used macros determined by the
+ *        major and minor version numbers.
  */
 
 #include <string.h>
@@ -84,6 +90,12 @@ static const l_int32 InterlacedJumps[] = {8, 8, 4, 2};
 #define EGifOpenFileHandle(a,b)  EGifOpenFileHandle(a)
 #endif  /* GIFLIB_MAJOR */
 
+    /* Basic interface changed again in 5.1 (!) */
+#if GIFLIB_MAJOR < 5 || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 0)
+#define DGifCloseFile(a,b)       DGifCloseFile(a)
+#define EGifCloseFile(a,b)       EGifCloseFile(a)
+#endif
+
 /*---------------------------------------------------------------------*
  *                       Reading gif from file                         *
  *---------------------------------------------------------------------*/
@@ -104,11 +116,18 @@ PIX             *pixd, *pixdi;
 PIXCMAP         *cmap;
 ColorMapObject  *gif_cmap;
 SavedImage       si;
+#if GIFLIB_MAJOR == 5 && GIFLIB_MINOR > 0
+int              giferr;
+#endif
 
     PROCNAME("pixReadStreamGif");
 
     if ((fd = fileno(fp)) < 0)
         return (PIX *)ERROR_PTR("invalid file descriptor", procName, NULL);
+#ifdef _WIN32
+    fd = _dup(fd);
+#endif /* _WIN32 */
+
 #ifndef _MSC_VER
     lseek(fd, 0, SEEK_SET);
 #else
@@ -121,12 +140,12 @@ SavedImage       si;
 
         /* Read all the data, but use only the first image found */
     if (DGifSlurp(gif) != GIF_OK) {
-        DGifCloseFile(gif);
+        DGifCloseFile(gif, &giferr);
         return (PIX *)ERROR_PTR("failed to read GIF data", procName, NULL);
     }
 
     if (gif->SavedImages == NULL) {
-        DGifCloseFile(gif);
+        DGifCloseFile(gif, &giferr);
         return (PIX *)ERROR_PTR("no images found in GIF", procName, NULL);
     }
 
@@ -134,12 +153,12 @@ SavedImage       si;
     w = si.ImageDesc.Width;
     h = si.ImageDesc.Height;
     if (w <= 0 || h <= 0) {
-        DGifCloseFile(gif);
+        DGifCloseFile(gif, &giferr);
         return (PIX *)ERROR_PTR("invalid image dimensions", procName, NULL);
     }
 
     if (si.RasterBits == NULL) {
-        DGifCloseFile(gif);
+        DGifCloseFile(gif, &giferr);
         return (PIX *)ERROR_PTR("no raster data in GIF", procName, NULL);
     }
 
@@ -151,7 +170,7 @@ SavedImage       si;
         gif_cmap = gif->SColorMap;
     } else {
             /* don't know where to take cmap from */
-        DGifCloseFile(gif);
+        DGifCloseFile(gif, &giferr);
         return (PIX *)ERROR_PTR("color map is missing", procName, NULL);
     }
 
@@ -164,8 +183,10 @@ SavedImage       si;
         d = 4;
     else
         d = 8;
-    if ((cmap = pixcmapCreate(d)) == NULL)
+    if ((cmap = pixcmapCreate(d)) == NULL) {
+        DGifCloseFile(gif, &giferr);
         return (PIX *)ERROR_PTR("cmap creation failed", procName, NULL);
+    }
 
     for (cindex = 0; cindex < ncolors; cindex++) {
         rval = gif_cmap->Colors[cindex].Red;
@@ -175,7 +196,7 @@ SavedImage       si;
     }
 
     if ((pixd = pixCreate(w, h, d)) == NULL) {
-        DGifCloseFile(gif);
+        DGifCloseFile(gif, &giferr);
         pixcmapDestroy(&cmap);
         return (PIX *)ERROR_PTR("failed to allocate pixd", procName, NULL);
     }
@@ -207,7 +228,7 @@ SavedImage       si;
         pixTransferAllData(pixd, &pixdi, 0, 0);
     }
 
-    DGifCloseFile(gif);
+    DGifCloseFile(gif, &giferr);
     return pixd;
 }
 
@@ -274,6 +295,9 @@ PIXCMAP         *cmap;
 GifFileType     *gif;
 ColorMapObject  *gif_cmap;
 GifByteType     *gif_line;
+#if GIFLIB_MAJOR == 5 && GIFLIB_MINOR > 0
+int              giferr;
+#endif
 
     PROCNAME("pixWriteStreamGif");
 
@@ -285,6 +309,9 @@ GifByteType     *gif_line;
 
     if ((fd = fileno(fp)) < 0)
         return ERROR_INT("invalid file descriptor", procName, 1);
+#ifdef _WIN32
+    fd = _dup(fd);
+#endif /* _WIN32 */
 
     d = pixGetDepth(pix);
     if (d == 32) {
@@ -356,14 +383,14 @@ GifByteType     *gif_line;
         != GIF_OK) {
         pixDestroy(&pixd);
         GifFreeMapObject(gif_cmap);
-        EGifCloseFile(gif);
+        EGifCloseFile(gif, &giferr);
         return ERROR_INT("failed to write screen description", procName, 1);
     }
     GifFreeMapObject(gif_cmap); /* not needed after this point */
 
     if (EGifPutImageDesc(gif, 0, 0, w, h, FALSE, NULL) != GIF_OK) {
         pixDestroy(&pixd);
-        EGifCloseFile(gif);
+        EGifCloseFile(gif, &giferr);
         return ERROR_INT("failed to image screen description", procName, 1);
     }
 
@@ -371,13 +398,13 @@ GifByteType     *gif_line;
     wpl = pixGetWpl(pixd);
     if (d != 1 && d != 2 && d != 4 && d != 8) {
         pixDestroy(&pixd);
-        EGifCloseFile(gif);
+        EGifCloseFile(gif, &giferr);
         return ERROR_INT("image depth is not in {1, 2, 4, 8}", procName, 1);
     }
 
     if ((gif_line = (GifByteType *)CALLOC(sizeof(GifByteType), w)) == NULL) {
         pixDestroy(&pixd);
-        EGifCloseFile(gif);
+        EGifCloseFile(gif, &giferr);
         return ERROR_INT("mem alloc fail for data line", procName, 1);
     }
 
@@ -406,7 +433,7 @@ GifByteType     *gif_line;
         if (EGifPutLine(gif, gif_line, w) != GIF_OK) {
             FREE(gif_line);
             pixDestroy(&pixd);
-            EGifCloseFile(gif);
+            EGifCloseFile(gif, &giferr);
             return ERROR_INT("failed to write data line into GIF", procName, 1);
         }
     }
@@ -422,7 +449,7 @@ GifByteType     *gif_line;
 
     FREE(gif_line);
     pixDestroy(&pixd);
-    EGifCloseFile(gif);
+    EGifCloseFile(gif, &giferr);
     return 0;
 }
 
@@ -477,7 +504,7 @@ PIX   *pix;
 
         /* Read back from the file */
     pix = pixRead(fname);
-    lept_rm(NULL, fname);
+    lept_rmfile(fname);
     lept_free(fname);
     if (!pix) L_ERROR("pix not read\n", procName);
     return pix;
@@ -520,7 +547,7 @@ char  *fname;
 
         /* Read back into memory */
     *pdata = l_binaryRead(fname, psize);
-    lept_rm(NULL, fname);
+    lept_rmfile(fname);
     lept_free(fname);
     return 0;
 }
