@@ -39,6 +39,7 @@
  *          NUMA        *numaWindowedMean()
  *          NUMA        *numaWindowedMeanSquare()
  *          l_int32      numaWindowedVariance()
+ *          NUMA        *numaWindowedMedian()
  *          NUMA        *numaConvertToInt()
  *
  *      Histogram generation and statistics
@@ -642,6 +643,58 @@ NUMA       *nav, *narv;  /* variance and square root of variance */
 }
 
 
+/*!
+ *  numaWindowedMedian()
+ *
+ *      Input:  nas
+ *              halfwin (half width of window over which the median is found)
+ *      Return: nad (after windowed median filtering), or null on error
+ *
+ *  Notes:
+ *      (1) The requested window has width = 2 * @halfwin + 1.
+ *      (2) If the input nas has less then 3 elements, just return a copy.
+ *      (3) If the requested filter is too large, it is reduced in size.
+ *      (4) We add a mirrored border of size @halfwin to each end of
+ *          the array to simplify the calculation by avoiding end-effects.
+ */
+NUMA *
+numaWindowedMedian(NUMA    *nas,
+                   l_int32  halfwin)
+{
+l_int32    i, n;
+l_float32  medval;
+NUMA      *na1, *na2, *nad;
+
+    PROCNAME("numaWindowedMedian");
+
+    if (!nas)
+        return (NUMA *)ERROR_PTR("nas not defined", procName, NULL);
+    if ((n = numaGetCount(nas)) < 3)
+        return numaCopy(nas);
+
+    if (halfwin > (n - 1) / 2) {
+        halfwin = (n - 1) / 2;
+        L_INFO("reducing filter to halfwin = %d\n", procName, halfwin);
+    }
+
+        /* Add a border to both ends */
+    na1 = numaAddSpecifiedBorder(nas, halfwin, halfwin, L_MIRRORED_BORDER);
+
+        /* Get the median value at the center of each window, corresponding
+         * to locations in the input nas. */
+    nad = numaCreate(n);
+    for (i = 0; i < n; i++) {
+        na2 = numaClipToInterval(na1, i, i + 2 * halfwin);
+        numaGetMedian(na2, &medval);
+        numaAddNumber(nad, medval);
+        numaDestroy(&na2);
+    }
+
+    numaDestroy(&na1);
+    return nad;
+}
+
+
 /*!
  *  numaConvertToInt()
  *
@@ -1027,7 +1080,8 @@ NUMA      *nad;
  *      Input:  na (an arbitrary set of numbers; not ordered and not
  *                  a histogram)
  *              maxbins (the maximum number of bins to be allowed in
- *                       the histogram; use 0 for consecutive integer bins)
+ *                       the histogram; use an integer larger than the
+ *                       largest number in @na for consecutive integer bins)
  *              &min (<optional return> min value of set)
  *              &max (<optional return> max value of set)
  *              &mean (<optional return> mean value of set)
