@@ -63,10 +63,11 @@ l_int32 freadHeaderPng(FILE *fp, l_int32 *pwidth, l_int32 *pheight,
 
 /* ----------------------------------------------------------------------*/
 
-l_int32 sreadHeaderPng(const l_uint8 *data, l_int32 *pwidth, l_int32 *pheight,
-                       l_int32 *pbps, l_int32 *pspp, l_int32 *piscmap)
+l_int32 readHeaderMemPng(const l_uint8 *data, size_t size, l_int32 *pwidth,
+                         l_int32 *pheight, l_int32 *pbps, l_int32 *pspp,
+                         l_int32 *piscmap)
 {
-    return ERROR_INT("function not present", "sreadHeaderPng", 1);
+    return ERROR_INT("function not present", "readHeaderMemPng", 1);
 }
 
 /* ----------------------------------------------------------------------*/
@@ -78,6 +79,20 @@ l_int32 fgetPngResolution(FILE *fp, l_int32 *pxres, l_int32 *pyres)
 
 /* ----------------------------------------------------------------------*/
 
+l_int32 isPngInterlaced(const char *filename, l_int32 *pinterlaced)
+{
+    return ERROR_INT("function not present", "isPngInterlaced", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
+l_int32 fgetPngColormapInfo(FILE *fp, PIXCMAP **pcmap, l_int32 *ptransparency)
+{
+    return ERROR_INT("function not present", "fgetPngColormapInfo", 1);
+}
+
+/* ----------------------------------------------------------------------*/
+
 l_int32 pixWritePng(const char *filename, PIX *pix, l_float32 gamma)
 {
     return ERROR_INT("function not present", "pixWritePng", 1);
@@ -102,7 +117,7 @@ l_int32 pixSetZlibCompression(PIX *pix, l_int32 compval)
 
 void l_pngSetReadStrip16To8(l_int32 flag)
 {
-    L_ERROR("function not present", "l_pngSetReadStrip16To8");
+    L_ERROR("function not present\n", "l_pngSetReadStrip16To8");
     return;
 }
 
