@@ -628,10 +628,10 @@ L_RDID    *did;
     if ((did = recogGetDid(recog)) == NULL)
         return (PIX *)ERROR_PTR("did not defined", procName, NULL);
 
+    bmf = bmfCreate(NULL, 8);
     pixs = pixScale(did->pixs, 4.0, 4.0);
     pix0 = pixAddBorderGeneral(pixs, 0, 0, 0, 40, 0);
     pix1 = pixConvertTo32(pix0);
-    bmf = bmfCreate("./fonts", 8);
     if (select == 0) {  /* Viterbi */
         natempl_s = did->natempl;
         nascore_s = did->nascore;
