@@ -62,7 +62,7 @@
  *         l_int32             recogShowAverageTemplates()
  *         PIX                *recogShowMatchesInRange()
  *         PIX                *recogShowMatch()
- *         l_int32             recogMakeBmf()
+ *         l_int32             recogResetBmf()
  *
  *      Static helpers
  *         static char        *l_charToString()
@@ -454,7 +454,7 @@ l_int32  w, h;
     PROCNAME("recogScaleCharacter");
 
     if (!recog)
-        return (PIX *)ERROR_PTR("pix not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("recog not defined", procName, NULL);
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
 
@@ -1180,7 +1180,7 @@ recogPadTrainingSet(L_RECOG  **precog,
                     l_int32    debug)
 {
 const char  *bootdir, *bootpattern, *bootpath;
-char        *boottext, *fontdir;
+char        *boottext;
 l_int32      i, k, min_nopad, npix, nclass, nboot, nsamp, nextra, ntoadd;
 l_int32      ave_height, targeth, setid, index, allclasses;
 l_int32     *lut;
@@ -1224,19 +1224,16 @@ L_RECOGA    *recoga;
      * input recog and return a generic boot recognizer that will
      * be run using scaling both width and height.
      * ----------------------------------------------------------*/
-    fontdir = (recog->fontdir) ? stringNew(recog->fontdir) : NULL;
     if (recog->samplenum < MIN_TOTAL_SAMPLES) {
         L_WARNING("too few samples in recog; using bootrecog only\n", procName);
         bootpath = recog->bootpath;
         L_INFO("boot path = %s\n", procName, bootpath);
         if ((pixaboot = pixaRead(bootpath)) == NULL)
             return ERROR_INT("pixaboot not read", procName, 1);
-        rec1 = recogCreateFromPixa(pixaboot, 20, 32, L_USE_AVERAGE, 100,
-                                    1, fontdir);
+        rec1 = recogCreateFromPixa(pixaboot, 20, 32, L_USE_AVERAGE, 100, 1);
         recogReplaceInRecoga(&recog, rec1);  /* destroys recog */
         *precog = rec1;
         pixaDestroy(&pixaboot);
-        FREE(fontdir);
         return 0;
     }
 
@@ -1262,8 +1259,7 @@ L_RECOGA    *recoga;
          * averages from a copy of the input recog, also scaled to h = 32. */
     if ((paa1 = pixaaReadFromFiles(bootdir, bootpattern, 0, 0)) == NULL)
         return ERROR_INT("boot recog files not found", procName, 1);
-    recoga = recogaCreateFromPixaa(paa1, 0, 32, L_USE_AVERAGE, 100, 1,
-                                   fontdir);
+    recoga = recogaCreateFromPixaa(paa1, 0, 32, L_USE_AVERAGE, 100, 1);
     pixaaDestroy(&paa1);
     if (!recoga)
         return ERROR_INT("recoga not made", procName, 1);
@@ -1271,12 +1267,10 @@ L_RECOGA    *recoga;
         /* The parameters of the input recog must match those of the
          * boot array.  Replace the input recog with a new one, that
          * uses the average templates for matching, scaled to h = 32. */
-    rec1 = recogCreateFromRecog(recog, 0, 32, L_USE_AVERAGE, 100, 1,
-                                fontdir);
+    rec1 = recogCreateFromRecog(recog, 0, 32, L_USE_AVERAGE, 100, 1);
     recogReplaceInRecoga(&recog, rec1);  /* destroys recog */
     *precog = rec1;
     recog = rec1;
-    FREE(fontdir);
 
         /* Now for each class in the recog, decide which recog in recoga
          * should be used to select samples for padding the recog. */
@@ -1285,12 +1279,13 @@ L_RECOGA    *recoga;
                               &naave, &pixadb);
 
     if (pixadb) {
+        lept_mkdir("recog");
         numaWriteStream(stderr, naset);
         numaWriteStream(stderr, naindex);
         numaWriteStream(stderr, nascore);
         numaWriteStream(stderr, naave);
         pix1 = pixaDisplayLinearly(pixadb, L_VERT, 1.0, 0, 20, 0, NULL);
-        pixWrite("/tmp/padmatch.png", pix1, IFF_PNG);
+        pixWrite("/tmp/recog/padmatch.png", pix1, IFF_PNG);
         pixDestroy(&pix1);
         pixaDestroy(&pixadb);
     }
@@ -2274,31 +2269,28 @@ PIXA   *pixa;
 
 
 /*!
- *  recogMakeBmf()
+ *  recogResetBmf()
  *
  *      Input:  recog
- *              fontdir (for bitmap fonts; typically "fonts")
  *              size  (of font; even integer between 4 and 20; default is 6)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      (1) This can be used to (re)set the size of the font used for
- *          debug labelling.
+ *      (1) Use this to reset the size of the font used for debug labelling.
  */
 l_int32
-recogMakeBmf(L_RECOG     *recog,
-             const char  *fontdir,
+recogResetBmf(L_RECOG     *recog,
              l_int32      size)
 {
-   PROCNAME("recogMakeBmf");
+   PROCNAME("recogResetBmf");
 
-   if (!recog || !fontdir)
-        return ERROR_INT("recog and fontdir not both defined", procName, 1);
+   if (!recog)
+        return ERROR_INT("recog not defined", procName, 1);
    if (size < 4 || size > 20 || (size % 2)) size = 6;
    if (size == recog->bmf_size) return 0;  /* no change */
 
    bmfDestroy(&recog->bmf);
-   recog->bmf = bmfCreate(fontdir, size);
+   recog->bmf = bmfCreate(NULL, size);
    recog->bmf_size = size;
    return 0;
 }
