@@ -134,10 +134,13 @@ L_REGPARAMS  *rp;
          * as a failure of the regression test. */
     rp->success = TRUE;
 
+        /* Make sure the regout subdirectory exists */
+    lept_mkdir("regout");
+
         /* Only open a stream to a temp file for the 'compare' case */
     if (argc == 1 || !strcmp(argv[1], "compare")) {
         rp->mode = L_REG_COMPARE;
-        rp->tempfile = genTempFilename("/tmp", "regtest_output.txt", 0, 1);
+        rp->tempfile = genPathname("/tmp/regout", "regtest_output.txt");
         rp->fp = fopenWriteStream(rp->tempfile, "wb");
         if (rp->fp == NULL) {
             rp->success = FALSE;
@@ -319,15 +322,15 @@ char     buf[256];
         /* Output on failure */
     if (fail == TRUE) {
             /* Write the two strings to file */
-        snprintf(buf, sizeof(buf), "/tmp/string1_%d_%lu", rp->index,
+        snprintf(buf, sizeof(buf), "/tmp/regout/string1_%d_%lu", rp->index,
                  (unsigned long)bytes1);
         l_binaryWrite(buf, "w", string1, bytes1);
-        snprintf(buf, sizeof(buf), "/tmp/string2_%d_%lu", rp->index,
+        snprintf(buf, sizeof(buf), "/tmp/regout/string2_%d_%lu", rp->index,
                  (unsigned long)bytes2);
         l_binaryWrite(buf, "w", string2, bytes2);
 
             /* Report comparison failure */
-        snprintf(buf, sizeof(buf), "/tmp/string*_%d_*", rp->index);
+        snprintf(buf, sizeof(buf), "/tmp/regout/string*_%d_*", rp->index);
         if (rp->fp) {
             fprintf(rp->fp,
                     "Failure in %s_reg: string comp for index %d; "
@@ -500,17 +503,23 @@ PIX     *pix1, *pix2;
 
         /* Generate the golden file name; used in 'generate' and 'compare' */
     splitPathAtExtension(localname, NULL, &ext);
-    snprintf(namebuf, sizeof(namebuf), "/tmp/golden/%s_golden.%d%s",
+    snprintf(namebuf, sizeof(namebuf), "/tmp/golden/%s_golden.%02d%s",
              rp->testname, rp->index, ext);
     FREE(ext);
 
         /* Generate mode.  No testing. */
     if (rp->mode == L_REG_GENERATE) {
             /* Save the file as a golden file */
-/*        fprintf(stderr, "%d: %s\n", rp->index, namebuf);  */
         ret = fileCopy(localname, namebuf);
-        if (!ret)
-            fprintf(stderr, "Copy: %s to %s\n", localname, namebuf);
+#if 0       /* Enable for details on writing of golden files */
+        if (!ret) {
+            char *local = genPathname(localname, NULL);
+            char *golden = genPathname(namebuf, NULL);
+            L_INFO("Copy: %s to %s\n", procName, local, golden);
+            FREE(local);
+            FREE(golden);
+        }
+#endif
         return ret;
     }
 
@@ -592,7 +601,7 @@ SARRAY  *sa;
     if (rp->mode != L_REG_COMPARE) return 0;
 
         /* Generate the golden file names */
-    snprintf(namebuf, sizeof(namebuf), "%s_golden.%d.", rp->testname, index1);
+    snprintf(namebuf, sizeof(namebuf), "%s_golden.%02d.", rp->testname, index1);
     sa = getSortedPathnamesInDirectory("/tmp/golden", namebuf, 0, 0);
     if (sarrayGetCount(sa) != 1) {
         sarrayDestroy(&sa);
@@ -603,7 +612,7 @@ SARRAY  *sa;
     name1 = sarrayGetString(sa, 0, L_COPY);
     sarrayDestroy(&sa);
 
-    snprintf(namebuf, sizeof(namebuf), "%s_golden.%d.", rp->testname, index2);
+    snprintf(namebuf, sizeof(namebuf), "%s_golden.%02d.", rp->testname, index2);
     sa = getSortedPathnamesInDirectory("/tmp/golden", namebuf, 0, 0);
     if (sarrayGetCount(sa) != 1) {
         sarrayDestroy(&sa);
@@ -648,9 +657,9 @@ SARRAY  *sa;
  *             (b) make a local file and "compare" with the golden file
  *             (c) make a local file and "display" the results
  *      (3) The canonical format of the local filename is:
- *            /tmp/<root of main name>.<count>.<format extension string>
+ *            /tmp/regout/<root of main name>.<count>.<format extension string>
  *          e.g., for scale_reg,
- *            /tmp/scale.0.png
+ *            /tmp/regout/scale.0.png
  */
 l_int32
 regTestWritePixAndCheck(L_REGPARAMS  *rp,
@@ -673,7 +682,7 @@ char   namebuf[256];
     }
 
         /* Generate the local file name */
-    snprintf(namebuf, sizeof(namebuf), "/tmp/%s.%d.%s", rp->testname,
+    snprintf(namebuf, sizeof(namebuf), "/tmp/regout/%s.%02d.%s", rp->testname,
              rp->index + 1, ImageFileFormatExtensions[format]);
 
         /* Write the local file */
