@@ -30,6 +30,7 @@
  *     High-level procedures for writing images to file:
  *        l_int32     pixaWriteFiles()
  *        l_int32     pixWrite()    [behavior depends on WRITE_AS_NAMED]
+ *        l_int32     pixWriteAutoFormat()
  *        l_int32     pixWriteStream()
  *        l_int32     pixWriteImpliedFormat()
  *        l_int32     pixWriteTempfile()
@@ -37,6 +38,7 @@
  *     Selection of output format if default is requested
  *        l_int32     pixChooseOutputFormat()
  *        l_int32     getImpliedFileFormat()
+ *        l_int32     pixGetAutoFormat()
  *        const char *getFormatExtension()
  *
  *     Write to memory
@@ -64,6 +66,7 @@
  *          tiff  (including most varieties of compression)
  *          gif
  *          webp
+ *          jp2 (jpeg2000)
  *  (3) Writing is supported through special interfaces:
  *          ps (PostScript, in psio1.c, psio2.c):
  *              level 1 (uncompressed)
@@ -72,7 +75,6 @@
  *          pdf (PDF, in pdfio.c):
  *              level 1 (g4 and dct encoding: requires tiff, jpg)
  *              level 2 (g4, dct and flate encoding: requires tiff, jpg, zlib)
- *  (4) No other output formats are supported, such as jp2 (jpeg2000)
  */
 
 #include <string.h>
@@ -242,8 +244,6 @@ FILE  *fp;
         return ERROR_INT("pix not defined", procName, 1);
     if (!filename)
         return ERROR_INT("filename not defined", procName, 1);
-    if (format == IFF_JP2)
-        return ERROR_INT("jp2 not supported", procName, 1);
 
     fname = genPathname(filename, NULL);
 
@@ -293,20 +293,38 @@ FILE  *fp;
         fclose(fp);
         return ERROR_INT("pix not written to stream", procName, 1);
     }
-
-        /* Close the stream except if GIF under windows, because
-         * EGifCloseFile() closes the windows file stream! */
-    if (format != IFF_GIF)
-        fclose(fp);
-#ifndef _WIN32
-    else  /* gif file */
-        fclose(fp);
-#endif  /* ! _WIN32 */
+    fclose(fp);
 
     return 0;
 }
 
 
+/*!
+ *  pixWriteAutoFormat()
+ *
+ *      Input:  filename
+ *              pix
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixWriteAutoFormat(const char  *filename,
+                   PIX         *pix)
+{
+l_int32  format;
+
+    PROCNAME("pixWriteAutoFormat");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!filename)
+        return ERROR_INT("filename not defined", procName, 1);
+
+    if (pixGetAutoFormat(pix, &format))
+        return ERROR_INT("auto format not returned", procName, 1);
+    return pixWrite(filename, pix, format);
+}
+
+
 /*!
  *  pixWriteStream()
  *
@@ -358,16 +376,16 @@ pixWriteStream(FILE    *fp,
         return pixWriteStreamPnm(fp, pix);
         break;
 
-    case IFF_GIF:
-        return pixWriteStreamGif(fp, pix);
-        break;
-
     case IFF_PS:
         return pixWriteStreamPS(fp, pix, NULL, 0, DEFAULT_SCALING);
         break;
 
+    case IFF_GIF:
+        return pixWriteStreamGif(fp, pix);
+        break;
+
     case IFF_JP2:
-        return ERROR_INT("jp2 format not supported", procName, 1);
+        return pixWriteStreamJp2k(fp, pix, 34, 0, 0, 0);
         break;
 
     case IFF_WEBP:
@@ -573,6 +591,51 @@ l_int32  format = IFF_UNKNOWN;
 }
 
 
+/*!
+ *  pixGetAutoFormat()
+ *
+ *      Input:  pix
+ *              &format
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The output formats are restricted to tiff, jpeg and png
+ *          because these are the most commonly used image formats and
+ *          the ones that are typically installed with leptonica.
+ *      (2) This decides what compression to use based on the pix.
+ *          It chooses tiff-g4 if 1 bpp without a colormap, jpeg with
+ *          quality 75 if grayscale, rgb or rgba (where it loses
+ *          the alpha layer), and lossless png for all other situations.
+ */
+l_int32
+pixGetAutoFormat(PIX      *pix,
+                 l_int32  *pformat)
+{
+l_int32   d;
+PIXCMAP  *cmap;
+
+    PROCNAME("pixGetAutoFormat");
+
+    if (!pformat)
+        return ERROR_INT("&format not defined", procName, 0);
+    *pformat = IFF_UNKNOWN;
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 0);
+
+    d = pixGetDepth(pix);
+    cmap = pixGetColormap(pix);
+    if (d == 1 && !cmap) {
+        *pformat = IFF_TIFF_G4;
+    } else if ((d == 8 && !cmap) || d == 24 || d == 32) {
+        *pformat = IFF_JFIF_JPEG;
+    } else {
+        *pformat = IFF_PNG;
+    }
+
+    return 0;
+}
+
+
 /*!
  *  getFormatExtension()
  *
@@ -671,7 +734,15 @@ l_int32  ret;
         break;
 
     case IFF_JP2:
-        return ERROR_INT("jp2 not supported", procName, 1);
+        ret = pixWriteMemJp2k(pdata, psize, pix, 34, 0, 0, 0);
+        break;
+
+    case IFF_WEBP:
+        ret = pixWriteMemWebP(pdata, psize, pix, 80, 0);
+        break;
+
+    case IFF_LPDF:
+        ret = pixWriteMemPdf(pdata, psize, pix, 0, NULL);
         break;
 
     case IFF_SPIX:
@@ -836,7 +907,7 @@ char            fullpath[_MAX_PATH];
         snprintf(buffer, L_BUF_SIZE, "/tmp/disp/write.%03d.jpg", index);
         pixWrite(buffer, pix2, IFF_JFIF_JPEG);
     }
-    tempname = stringNew(buffer);
+    tempname = genPathname(buffer, NULL);
 
 #ifndef _WIN32
 
@@ -939,7 +1010,7 @@ char     fullpath[_MAX_PATH];
     FREE(tail);
 #endif  /* _WIN32 */
 
-    ignore = system(buffer);
+    ignore = system(buffer);  /* gthumb || i_view32.exe */
     return 0;
 }
 
